//!	EOS_AnalogInput.h


//--------------------------- inclusion guards ----------------------
#ifndef _EOS_AnalogInput_h_
#define _EOS_AnalogInput_h_


//--------------------------- include section -----------------------
#include "EOS.h"


//--------------------------- class definition ----------------------
//! this class represents an abstraction of an analog input
//! \code
//! class CPinA0 :
//!     public CEOS_AnalogInput
//! \endcode
class CEOS_AnalogInput
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CEOS_AnalogInput);


//--------------------------- public access -------------------------
public:
    // operator unsigned int ----------------------------------------
    //! converts the readAnalog() value into an int value
    inline operator unsigned int ()
    {
        return readAnalog();
    }


//--------------------------- public virtual access -----------------
public:
    // toAnalogInput ------------------------------------------------
    //! switches the data direction registers to be an analog input\n
    //! This is a pure virtual function and needs to be implemented
    //! in the derived classes according the µC datasheet
    virtual void toAnalogInput() = 0;

    // readAnalog ---------------------------------------------------
    //! reads from the analog input\n
    //! This is a pure virtual function and needs to be implemented
    //! in the derived classes according the µC datasheet
    virtual unsigned int readAnalog() = 0;

    // getAnalogInputResolution -------------------------------------
    //! \return the max value, which might return by the ADC
    //! This is a pure virtual function and needs to be implemented
    //! in the derived classes according the µC datasheet
    virtual unsigned int getAnalogInputResolution() const = 0;
};


#endif
