//!	EOS_AnalogOutput.h


//--------------------------- inclusion guards ----------------------
#ifndef _EOS_AnalogOutput_h_
#define _EOS_AnalogOutput_h_


//--------------------------- include section -----------------------
#include "types.h"
#include "EOS_AnalogInput.h"


//--------------------------- class definition ----------------------
//! this class represents an abstraction of an analog output
//! \code
//! class CPin3 :
//!     public CEOS_AnalogOutput
//! \endcode
class CEOS_AnalogOutput
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CEOS_AnalogOutput);


//--------------------------- public access -------------------------
public:
    // operator = ---------------------------------------------------
    //! sets the output with the given analog value
    inline void operator = (unsigned int nValue)
    {
        writeAnalog(nValue);
    }

    // operator = ---------------------------------------------------
    //! sets the output with the value read from the analog input
    inline void operator = (CEOS_AnalogInput& EOS_AnalogInput)
    {
        writeAnalog(EOS_AnalogInput.readAnalog());
    }


//--------------------------- public virtual access -----------------
public:
    // toAnalogOutput -----------------------------------------------
    //! switches the data direction registers to be an analog output\n
    //! This is a pure virtual function and needs to be implemented
    //! in the derived classes according the µC datasheet
    virtual void toAnalogOutput() = 0;
	
    // writeAnalog --------------------------------------------------
    //! sets the output to deliver the given value\n
    //! This is a pure virtual function and needs to be implemented
    //! in the derived classes according the µC datasheet
    //! \param nValue shall be between 0 and getAnalogOutputResolution()
    virtual void writeAnalog(unsigned int nValue) = 0;

    // getAnalogOutputResolution ------------------------------------
    //! \return the max value, which might return by the ADC
    //! This is a pure virtual function and needs to be implemented
    //! in the derived classes according the µC datasheet
    virtual unsigned int getAnalogOutputResolution() const = 0;
};


#endif
