//!	EOS_EventHandler.h


//--------------------------- inclusion guards ----------------------
#ifndef _EOS_EventHandler_h_
#define _EOS_EventHandler_h_


//--------------------------- include section -----------------------
#include "EOS_HandlerBase.h"
#include "EOS_New.h"


//--------------------------- class definition ----------------------
//! A CEOS_EventHandler is an internally used class, storing the
//! pointer to a member function of a class, which shall be executed
//! when requested.
//! This class is a template class with an object type and 10
//! arguments, specialized template versions exists down to 1
//! argument and even no argument
template <class T_OBJECT,
          typename A1=void,
          typename A2=void,
          typename A3=void,
          typename A4=void,
          typename A5=void,
          typename A6=void,
          typename A7=void,
          typename A8=void,
          typename A9=void,
          typename A10=void>
class CEOS_EventHandler :
        public CEOS_HandlerBase<A1,A2,A3,A4,A5,A6,A7,A8,A9,A10>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! stores the pointer to the object with the member function
    //! with the right number and type of arguments for later calling
    CEOS_EventHandler(T_OBJECT* pObject,
                      void(T_OBJECT::* pFunc)(A1,A2,A3,A4,A5,A6,A7,A8,A9,A10)) :
        m_pObject(pObject),
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    //! \internal
    //! calls the member function of the stored object handing over
    //! all the arguments
    inline virtual void onExecute(A1 arg1,
                                  A2 arg2,
                                  A3 arg3,
                                  A4 arg4,
                                  A5 arg5,
                                  A6 arg6,
                                  A7 arg7,
                                  A8 arg8,
                                  A9 arg9,
                                  A10 arg10)
    {
        (m_pObject->* m_pFunc)(arg1,
                               arg2,
                               arg3,
                               arg4,
                               arg5,
                               arg6,
                               arg7,
                               arg8,
                               arg9,
                               arg10);
    }

//--------------------------- protected data ------------------------
protected:
    // m_pObject ----------------------------------------------------
    //! \internal
    //! pointer to the object, which contains the member function
    T_OBJECT* m_pObject;

    // m_pFunc ------------------------------------------------------
    //! \internal
    //! pointer to the member function
    void(T_OBJECT::* m_pFunc)(A1,A2,A3,A4,A5,A6,A7,A8,A9,A10);
};


//###################################################################
//########################### C #####################################
//###################################################################

//########################### spezialiced template ##################
//--------------------------- class definition ----------------------
template <typename A1,
          typename A2,
          typename A3,
          typename A4,
          typename A5,
          typename A6,
          typename A7,
          typename A8,
          typename A9,
          typename A10>
class CEOS_EventHandler<void,A1,A2,A3,A4,A5,A6,A7,A8,A9,A10> :
        public CEOS_HandlerBase<A1,A2,A3,A4,A5,A6,A7,A8,A9,A10>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_EventHandler(void(*pFunc)(A1,A2,A3,A4,A5,A6,A7,A8,A9)) :
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    inline virtual void onExecute(A1 arg1,
                                  A2 arg2,
                                  A3 arg3,
                                  A4 arg4,
                                  A5 arg5,
                                  A6 arg6,
                                  A7 arg7,
                                  A8 arg8,
                                  A9 arg9)
    {
        (m_pFunc)(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9);
    }

//--------------------------- protected data ------------------------
protected:
    // m_pFunc ------------------------------------------------------
    //! pointer to the C function
    void(*m_pFunc)(A1,A2,A3,A4,A5,A6,A7,A8,A9);
};


//########################### spezialiced template ##################
//--------------------------- class definition ----------------------
template <typename A1,
          typename A2,
          typename A3,
          typename A4,
          typename A5,
          typename A6,
          typename A7,
          typename A8,
          typename A9>
class CEOS_EventHandler<void,A1,A2,A3,A4,A5,A6,A7,A8,A9> :
        public CEOS_HandlerBase<A1,A2,A3,A4,A5,A6,A7,A8,A9>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_EventHandler(void(*pFunc)(A1,A2,A3,A4,A5,A6,A7,A8,A9)) :
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    inline virtual void onExecute(A1 arg1,
                                  A2 arg2,
                                  A3 arg3,
                                  A4 arg4,
                                  A5 arg5,
                                  A6 arg6,
                                  A7 arg7,
                                  A8 arg8,
                                  A9 arg9)
    {
        (m_pFunc)(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9);
    }

//--------------------------- protected data ------------------------
protected:
    // m_pFunc ------------------------------------------------------
    //! pointer to the C function
    void(*m_pFunc)(A1,A2,A3,A4,A5,A6,A7,A8,A9);
};


//########################### spezialiced template ##################
//--------------------------- class definition ----------------------
template <typename A1,
          typename A2,
          typename A3,
          typename A4,
          typename A5,
          typename A6,
          typename A7,
          typename A8>
class CEOS_EventHandler<void,A1,A2,A3,A4,A5,A6,A7,A8> :
        public CEOS_HandlerBase<A1,A2,A3,A4,A5,A6,A7,A8>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_EventHandler(void(*pFunc)(A1,A2,A3,A4,A5,A6,A7,A8)) :
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    inline virtual void onExecute(A1 arg1,
                                  A2 arg2,
                                  A3 arg3,
                                  A4 arg4,
                                  A5 arg5,
                                  A6 arg6,
                                  A7 arg7,
                                  A8 arg8)
    {
        (m_pFunc)(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8);
    }

//--------------------------- protected data ------------------------
protected:
    // m_pFunc ------------------------------------------------------
    //! pointer to the C function
    void(*m_pFunc)(A1,A2,A3,A4,A5,A6,A7,A8);
};


//########################### spezialiced template ##################
//--------------------------- class definition ----------------------
template <typename A1,
          typename A2,
          typename A3,
          typename A4,
          typename A5,
          typename A6,
          typename A7>
class CEOS_EventHandler<void,A1,A2,A3,A4,A5,A6,A7> :
        public CEOS_HandlerBase<A1,A2,A3,A4,A5,A6,A7>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_EventHandler(void(*pFunc)(A1,A2,A3,A4,A5,A6,A7)) :
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    inline virtual void onExecute(A1 arg1,
                                  A2 arg2,
                                  A3 arg3,
                                  A4 arg4,
                                  A5 arg5,
                                  A6 arg6,
                                  A7 arg7)
    {
        (m_pFunc)(arg1,arg2,arg3,arg4,arg5,arg6,arg7);
    }

//--------------------------- protected data ------------------------
protected:
    // m_pFunc ------------------------------------------------------
    //! pointer to the C function
    void(*m_pFunc)(A1,A2,A3,A4,A5,A6,A7);
};


//########################### spezialiced template ##################
//--------------------------- class definition ----------------------
template <typename A1,
          typename A2,
          typename A3,
          typename A4,
          typename A5,
          typename A6>
class CEOS_EventHandler<void,A1,A2,A3,A4,A5,A6> :
        public CEOS_HandlerBase<A1,A2,A3,A4,A5,A6>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_EventHandler(void(*pFunc)(A1,A2,A3,A4,A5,A6)) :
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    inline virtual void onExecute(A1 arg1,
                                  A2 arg2,
                                  A3 arg3,
                                  A4 arg4,
                                  A5 arg5,
                                  A6 arg6)
    {
        (m_pFunc)(arg1,arg2,arg3,arg4,arg5,arg6);
    }

//--------------------------- protected data ------------------------
protected:
    // m_pFunc ------------------------------------------------------
    //! pointer to the C function
    void(*m_pFunc)(A1,A2,A3,A4,A5,A6);
};


//########################### spezialiced template ##################
//--------------------------- class definition ----------------------
template <typename A1,
          typename A2,
          typename A3,
          typename A4,
          typename A5>
class CEOS_EventHandler<void,A1,A2,A3,A4,A5> :
        public CEOS_HandlerBase<A1,A2,A3,A4,A5>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_EventHandler(void(*pFunc)(A1,A2,A3,A4,A5)) :
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    inline virtual void onExecute(A1 arg1,
                                  A2 arg2,
                                  A3 arg3,
                                  A4 arg4,
                                  A5 arg5)
    {
        (m_pFunc)(arg1,arg2,arg3,arg4,arg5);
    }

//--------------------------- protected data ------------------------
protected:
    // m_pFunc ------------------------------------------------------
    //! pointer to the C function
    void(*m_pFunc)(A1,A2,A3,A4,A5);
};


//########################### spezialiced template ##################
//--------------------------- class definition ----------------------
template <typename A1,
          typename A2,
          typename A3,
          typename A4>
class CEOS_EventHandler<void,A1,A2,A3,A4> :
        public CEOS_HandlerBase<A1,A2,A3,A4>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_EventHandler(void(*pFunc)(A1,A2,A3,A4)) :
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    inline virtual void onExecute(A1 arg1,
                                  A2 arg2,
                                  A3 arg3,
                                  A4 arg4)
    {
        (m_pFunc)(arg1,arg2,arg3,arg4);
    }

//--------------------------- protected data ------------------------
protected:
    // m_pFunc ------------------------------------------------------
    //! pointer to the C function
    void(*m_pFunc)(A1,A2,A3,A4);
};


//########################### spezialiced template ##################
//--------------------------- class definition ----------------------
template <typename A1,
          typename A2,
          typename A3>
class CEOS_EventHandler<void,A1,A2,A3> :
        public CEOS_HandlerBase<A1,A2,A3>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_EventHandler(void(*pFunc)(A1,A2,A3)) :
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    inline virtual void onExecute(A1 arg1,
                                  A2 arg2,
                                  A3 arg3)
    {
        (m_pFunc)(arg1,arg2,arg3);
    }

//--------------------------- protected data ------------------------
protected:
    // m_pFunc ------------------------------------------------------
    //! pointer to the C function
    void(*m_pFunc)(A1,A2,A3);
};


//########################### spezialiced template ##################
//--------------------------- class definition ----------------------
template <typename A1,
          typename A2>
class CEOS_EventHandler<void,A1,A2> :
        public CEOS_HandlerBase<A1,A2>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_EventHandler(void(*pFunc)(A1,A2)) :
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    inline virtual void onExecute(A1 arg1,
                                  A2 arg2)
    {
        (m_pFunc)(arg1,arg2);
    }

//--------------------------- protected data ------------------------
protected:
    // m_pFunc ------------------------------------------------------
    //! pointer to the C function
    void(*m_pFunc)(A1,A2);
};


//########################### spezialiced template ##################
//--------------------------- class definition ----------------------
template <typename A1>
class CEOS_EventHandler<void,A1> :
        public CEOS_HandlerBase<A1>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_EventHandler(void(*pFunc)(A1)) :
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    inline virtual void onExecute(A1 arg1)
    {
        (m_pFunc)(arg1);
    }

//--------------------------- protected data ------------------------
protected:
    // m_pFunc ------------------------------------------------------
    //! pointer to the C function
    void(*m_pFunc)(A1);
};


//########################### spezialiced template ##################
//--------------------------- class definition ----------------------
template <>
class CEOS_EventHandler<void> :
        public CEOS_HandlerBase<>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_EventHandler(void(*pFunc)()) :
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    inline virtual void onExecute()
    {
        (m_pFunc)();
    }

//--------------------------- protected data ------------------------
protected:
    // m_pFunc ------------------------------------------------------
    //! pointer to the C function
    void(*m_pFunc)();
};


//###################################################################
//########################### C++ ###################################
//###################################################################

//########################### spezialiced template ##################
//--------------------------- class definition ----------------------
template <class T_OBJECT,
          typename A1,
          typename A2,
          typename A3,
          typename A4,
          typename A5,
          typename A6,
          typename A7,
          typename A8,
          typename A9>
class CEOS_EventHandler<T_OBJECT,A1,A2,A3,A4,A5,A6,A7,A8,A9> :
        public CEOS_HandlerBase<A1,A2,A3,A4,A5,A6,A7,A8,A9>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_EventHandler(T_OBJECT* pObject,
                      void(T_OBJECT::* pFunc)(A1,A2,A3,A4,A5,A6,A7,A8,A9)) :
        m_pObject(pObject),
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    inline virtual void onExecute(A1 arg1,
                                  A2 arg2,
                                  A3 arg3,
                                  A4 arg4,
                                  A5 arg5,
                                  A6 arg6,
                                  A7 arg7,
                                  A8 arg8,
                                  A9 arg9)
    {
        (m_pObject->* m_pFunc)(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9);
    }

//--------------------------- protected data ------------------------
protected:
    // m_pObject ----------------------------------------------------
    //! pointer to the object, which contains the member function
    T_OBJECT* m_pObject;

    // m_pFunc ------------------------------------------------------
    //! pointer to the member function
    void(T_OBJECT::* m_pFunc)(A1,A2,A3,A4,A5,A6,A7,A8,A9);
};


//########################### spezialiced template ##################
//--------------------------- class definition ----------------------
template <class T_OBJECT,
          typename A1,
          typename A2,
          typename A3,
          typename A4,
          typename A5,
          typename A6,
          typename A7,
          typename A8>
class CEOS_EventHandler<T_OBJECT,A1,A2,A3,A4,A5,A6,A7,A8> :
        public CEOS_HandlerBase<A1,A2,A3,A4,A5,A6,A7,A8>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_EventHandler(T_OBJECT* pObject,
                      void(T_OBJECT::* pFunc)(A1,A2,A3,A4,A5,A6,A7,A8)) :
        m_pObject(pObject),
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    inline virtual void onExecute(A1 arg1,
                                  A2 arg2,
                                  A3 arg3,
                                  A4 arg4,
                                  A5 arg5,
                                  A6 arg6,
                                  A7 arg7,
                                  A8 arg8)
    {
        (m_pObject->* m_pFunc)(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8);
    }

//--------------------------- protected data ------------------------
protected:
    // m_pObject ----------------------------------------------------
    //! pointer to the object, which contains the member function
    T_OBJECT* m_pObject;

    // m_pFunc ------------------------------------------------------
    //! pointer to the member function
    void(T_OBJECT::* m_pFunc)(A1,A2,A3,A4,A5,A6,A7,A8);
};


//########################### spezialiced template ##################
//--------------------------- class definition ----------------------
template <class T_OBJECT,
          typename A1,
          typename A2,
          typename A3,
          typename A4,
          typename A5,
          typename A6,
          typename A7>
class CEOS_EventHandler<T_OBJECT,A1,A2,A3,A4,A5,A6,A7> :
        public CEOS_HandlerBase<A1,A2,A3,A4,A5,A6,A7>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_EventHandler(T_OBJECT* pObject,
                      void(T_OBJECT::* pFunc)(A1,A2,A3,A4,A5,A6,A7)) :
        m_pObject(pObject),
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    inline virtual void onExecute(A1 arg1,
                                  A2 arg2,
                                  A3 arg3,
                                  A4 arg4,
                                  A5 arg5,
                                  A6 arg6,
                                  A7 arg7)
    {
        (m_pObject->* m_pFunc)(arg1,arg2,arg3,arg4,arg5,arg6,arg7);
    }

//--------------------------- protected data ------------------------
protected:
    // m_pObject ----------------------------------------------------
    //! pointer to the object, which contains the member function
    T_OBJECT* m_pObject;

    // m_pFunc ------------------------------------------------------
    //! pointer to the member function
    void(T_OBJECT::* m_pFunc)(A1,A2,A3,A4,A5,A6,A7);
};


//########################### spezialiced template ##################
//--------------------------- class definition ----------------------
template <class T_OBJECT,
          typename A1,
          typename A2,
          typename A3,
          typename A4,
          typename A5,
          typename A6>
class CEOS_EventHandler<T_OBJECT,A1,A2,A3,A4,A5,A6> :
        public CEOS_HandlerBase<A1,A2,A3,A4,A5,A6>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_EventHandler(T_OBJECT* pObject,
                      void(T_OBJECT::* pFunc)(A1,A2,A3,A4,A5,A6)) :
        m_pObject(pObject),
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    inline virtual void onExecute(A1 arg1,
                                  A2 arg2,
                                  A3 arg3,
                                  A4 arg4,
                                  A5 arg5,
                                  A6 arg6)
    {
        (m_pObject->* m_pFunc)(arg1,arg2,arg3,arg4,arg5,arg6);
    }

//--------------------------- protected data ------------------------
protected:
    // m_pObject ----------------------------------------------------
    //! pointer to the object, which contains the member function
    T_OBJECT* m_pObject;

    // m_pFunc ------------------------------------------------------
    //! pointer to the member function
    void(T_OBJECT::* m_pFunc)(A1,A2,A3,A4,A5,A6);
};


//########################### spezialiced template ##################
//--------------------------- class definition ----------------------
template <class T_OBJECT,
          typename A1,
          typename A2,
          typename A3,
          typename A4,
          typename A5>
class CEOS_EventHandler<T_OBJECT,A1,A2,A3,A4,A5> :
        public CEOS_HandlerBase<A1,A2,A3,A4,A5>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_EventHandler(T_OBJECT* pObject,
                      void(T_OBJECT::* pFunc)(A1,A2,A3,A4,A5)) :
        m_pObject(pObject),
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    inline virtual void onExecute(A1 arg1,
                                  A2 arg2,
                                  A3 arg3,
                                  A4 arg4,
                                  A5 arg5)
    {
        (m_pObject->* m_pFunc)(arg1,arg2,arg3,arg4,arg5);
    }

//--------------------------- protected data ------------------------
protected:
    // m_pObject ----------------------------------------------------
    //! pointer to the object, which contains the member function
    T_OBJECT* m_pObject;

    // m_pFunc ------------------------------------------------------
    //! pointer to the member function
    void(T_OBJECT::* m_pFunc)(A1,A2,A3,A4,A5);
};


//########################### spezialiced template ##################
//--------------------------- class definition ----------------------
template <class T_OBJECT,
          typename A1,
          typename A2,
          typename A3,
          typename A4>
class CEOS_EventHandler<T_OBJECT,A1,A2,A3,A4> :
        public CEOS_HandlerBase<A1,A2,A3,A4>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_EventHandler(T_OBJECT* pObject,
                      void(T_OBJECT::* pFunc)(A1,A2,A3,A4)) :
        m_pObject(pObject),
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    inline virtual void onExecute(A1 arg1,
                                  A2 arg2,
                                  A3 arg3,
                                  A4 arg4)
    {
        (m_pObject->* m_pFunc)(arg1,arg2,arg3,arg4);
    }

//--------------------------- protected data ------------------------
protected:
    // m_pObject ----------------------------------------------------
    //! pointer to the object, which contains the member function
    T_OBJECT* m_pObject;

    // m_pFunc ------------------------------------------------------
    //! pointer to the member function
    void(T_OBJECT::* m_pFunc)(A1,A2,A3,A4);
};


//########################### spezialiced template ##################
//--------------------------- class definition ----------------------
template <class T_OBJECT,
          typename A1,
          typename A2,
          typename A3>
class CEOS_EventHandler<T_OBJECT,A1,A2,A3> :
        public CEOS_HandlerBase<A1,A2,A3>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_EventHandler(T_OBJECT* pObject,
                      void(T_OBJECT::* pFunc)(A1,A2,A3)) :
        m_pObject(pObject),
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    inline virtual void onExecute(A1 arg1,
                                  A2 arg2,
                                  A3 arg3)
    {
        (m_pObject->* m_pFunc)(arg1,arg2,arg3);
    }

//--------------------------- protected data ------------------------
protected:
    // m_pObject ----------------------------------------------------
    //! pointer to the object, which contains the member function
    T_OBJECT* m_pObject;

    // m_pFunc ------------------------------------------------------
    //! pointer to the member function
    void(T_OBJECT::* m_pFunc)(A1,A2,A3);
};


//########################### spezialiced template ##################
//--------------------------- class definition ----------------------
template <class T_OBJECT,
          typename A1,
          typename A2>
class CEOS_EventHandler<T_OBJECT,A1,A2> :
        public CEOS_HandlerBase<A1,A2>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_EventHandler(T_OBJECT* pObject,
                      void(T_OBJECT::* pFunc)(A1,A2)) :
        m_pObject(pObject),
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    inline virtual void onExecute(A1 arg1,
                                  A2 arg2)
    {
        (m_pObject->* m_pFunc)(arg1,arg2);
    }

//--------------------------- protected data ------------------------
protected:
    // m_pObject ----------------------------------------------------
    //! pointer to the object, which contains the member function
    T_OBJECT* m_pObject;

    // m_pFunc ------------------------------------------------------
    //! pointer to the member function
    void(T_OBJECT::* m_pFunc)(A1,A2);
};


//########################### spezialiced template ##################
//--------------------------- class definition ----------------------
template <class T_OBJECT,
          typename A1>
class CEOS_EventHandler<T_OBJECT,A1> :
        public CEOS_HandlerBase<A1>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_EventHandler(T_OBJECT* pObject,
                      void(T_OBJECT::* pFunc)(A1)) :
        m_pObject(pObject),
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    inline virtual void onExecute(A1 arg1)
    {
        (m_pObject->* m_pFunc)(arg1);
    }

//--------------------------- protected data ------------------------
protected:
    // m_pObject ----------------------------------------------------
    //! pointer to the object, which contains the member function
    T_OBJECT* m_pObject;

    // m_pFunc ------------------------------------------------------
    //! pointer to the member function
    void(T_OBJECT::* m_pFunc)(A1);
};


//########################### spezialiced template ##################
//--------------------------- class definition ----------------------
template <class T_OBJECT>
class CEOS_EventHandler<T_OBJECT> :
        public CEOS_HandlerBase<>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CEOS_EventHandler(T_OBJECT* pObject,
                      void(T_OBJECT::* pFunc)()) :
        m_pObject(pObject),
        m_pFunc(pFunc)
    {
        // Nothing to do
    }

//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    inline virtual void onExecute()
    {
        (m_pObject->* m_pFunc)();
    }

//--------------------------- protected data ------------------------
protected:
    // m_pObject ----------------------------------------------------
    //! pointer to the object, which contains the member function
    T_OBJECT* m_pObject;

    // m_pFunc ------------------------------------------------------
    //! pointer to the member function
    void(T_OBJECT::* m_pFunc)();
};

#endif
