//!	EOS_FrequencyOutput.h


//--------------------------- inclusion guards ----------------------
#ifndef _EOS_FrequencyOutput_h_
#define _EOS_FrequencyOutput_h_


//--------------------------- include section -----------------------
#include "types.h"
#include "EOS_AnalogOutput.h"


//--------------------------- class definition ----------------------
//! this class represents an abstraction of an analog output capable
//! to perform frequency output
//! \code
//! class CPin :
//!     public CEOS_FrequencyOutput
//! \endcode
class CEOS_FrequencyOutput
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CEOS_FrequencyOutput);


//--------------------------- public virtual access -----------------
public:
    // toFrequencyOutput --------------------------------------------
    //! changes the output to frequency mode
    //! \param nFrequency is the requested frequency
    //! \code
    //! // set the frequency to 440 Hz
    //! if (GPIO1.setFrequency(100,200) == 0)
    //! {
    //!     // not possible
    //! }
    //! \endcode
    virtual void toFrequencyOutput() = 0;

    // setFrequency -------------------------------------------------
    //! sets the requested frequency
    //! \param nFrequency is the requested frequency
    //! \code
    //! // set the frequency to 440 Hz
    //! GPIO1.setFrequency(440);
    //! \endcode
    virtual void setFrequency(unsigned int nFrequency) = 0;
};


#endif
