//!	EOS_PullDownResistor.h


//--------------------------- inclusion guards ----------------------
#ifndef _EOS_PullDownResistor_h_
#define _EOS_PullDownResistor_h_


//--------------------------- include section -----------------------
#include "types.h"


//--------------------------- class definition ----------------------
//! this class represents an abstraction of a pin, that has a
//! switchable pull down pin
//! \code
//! class CPin :
//!     public CEOS_PullDownResistor
//! \endcode
class CEOS_PullDownResistor
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CEOS_PullDownResistor);


//--------------------------- public virtual access -----------------
public:
    // setPullDownResistor ------------------------------------------
    //! switches on/off the pull down resistor on this pin
    //! \param bEnabled is true for switching on the resistor, false
    //! otherwise
    //! \code
    //! GPIO13.setPullDownResistor(true);
    //! \endcode
    virtual void setPullDownResistor(bool bEnabled) = 0;

    // enablePullDownResistor ---------------------------------------
    //! switches on the pull down resistor on this pin
    //! \code
    //! GPIO13.enablePullDown();
    //! \endcode
    inline void enablePullDownResistor()
    {
        setPullDownResistor(true);
    }

    // disablePullDownResistor --------------------------------------
    //! switches off the pull down resistor on this pin
    //! \code
    //! GPIO13.disablePullDown();
    //! \endcode
    inline void disablePullDownResistor()
    {
        setPullDownResistor(false);
    }
};


#endif
