//!	EOS_Task.h


//--------------------------- inclusion guards ----------------------
#ifndef _EOS_Task_h_
#define _EOS_Task_h_


//--------------------------- include section -----------------------
#include "EOS_TaskHandler.h"
#include "EOS_New.h"
#include "EOS_Delete.h"


//--------------------------- class definition ----------------------
//! A CEOS_Task is the base class for all derived task classes.
//! This class is used internally only by the derived classes
class CEOS_Task :
        public CEOS_New,
        public CEOS_Delete
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! generates a CEOS_TaskHandler, which will execute a static
    //! function
    CEOS_Task(void(pFunc)())
    {
        m_pHandler = new CEOS_TaskHandler<void>(pFunc);
    }

    // constructor --------------------------------------------------
    //! \internal
    //! generates a CEOS_TaskHandler, which will execute the member
    //! function
    template <class T_OBJECT>
    CEOS_Task(T_OBJECT* pObject,
              void(T_OBJECT::* pFunc)())
    {
        m_pHandler = new CEOS_TaskHandler<T_OBJECT>(pObject,pFunc);
    }

    // destructor ---------------------------------------------------
    //! \internal
    //! when the task is deleted (no longer used by the scheduler)
    //! the CEOS_TaskHandler object is also from no longer use
    //! this function must be virtual!
    virtual ~CEOS_Task()
    {
        delete m_pHandler;
    }

    NO_COPY_AND_ASSIGNMENT(CEOS_Task);


//--------------------------- public access -------------------------
public:
    // execute ------------------------------------------------------
    //! \internal
    //! let the handler do it's task
    void execute()
    {
        m_pHandler->onExecute();
    }


//--------------------------- protected data ------------------------
protected:
    // m_pHandler ---------------------------------------------------
    //! \internal
    //! pointer to the CEOS_HandlerBase (the CEOS_TaskHandler is of
    //! an unknown number of template arguments)
    CEOS_HandlerBase<>* m_pHandler;
};


#endif
