//!	EOS_TxStream.cpp


//--------------------------- include section -----------------------
#include "EOS_TxStream.h"


//--------------------------- private defines -----------------------
    // PRINT_BUF_LEN_INT --------------------------------------------
    // max can be 4294967296 (10 digits)
    #define PRINT_BUF_LEN_INT (10+1)


//-------------------------------------------------------------------
// Constructor
CEOS_TxStream::CEOS_TxStream()
{
    reset();
}


//-------------------------------------------------------------------
// print
void CEOS_TxStream::print(const char* pText)
{
    while (*pText != '\0')
    {
        write(*pText);
        pText++;
    }
}


//-------------------------------------------------------------------
// length
size_t CEOS_TxStream::length(const char* pText) const
{
    size_t nLength = 0;
    while (*pText != '\0')
    {
        nLength++;
        pText++;
    }
    return nLength;
}


//-------------------------------------------------------------------
// print
void CEOS_TxStream::print(unsigned int nValue,
                          uint8 nBase,
                          char chLetterBase)
{
    uint8 nLength = 1;

    unsigned int nShift = nValue;
    while (nShift >= nBase)
    {
        nShift /= nBase;
        nLength++;
    }

    do
    {
        unsigned int nShift = nValue;
        unsigned int nReduction = 1;
        for (uint8 i = 1; i < nLength; i++)
        {
            nShift /= nBase;
            nReduction *= nBase;
        }

        nValue -= nReduction * nShift;

        if (nShift >= 10)
        {
            write(chLetterBase + nShift - 10);
        }
        else
        {
            write('0' + nShift);
        }
        nLength--;
    } while (nLength > 0);
}


//-------------------------------------------------------------------
// length
size_t CEOS_TxStream::length(unsigned int nValue,
                             uint8 nBase) const
{
    if (nValue == 0)
    {
        return 1;
    }

    size_t nLength = 0;

    // Get the length
    while (nValue > 0)
    {
        // Next digit
        nValue /= nBase;
        nLength++;
    };

    return nLength;
}



//-------------------------------------------------------------------
// print
void CEOS_TxStream::print(const char* strText,
                          size_t nWidth,
                          UFormat uFormat)
{
    size_t nLength = length(strText);

    if (nWidth < nLength)
    {
        nWidth = 0;
    }
    else
    {
        nWidth -= nLength;
    }

    // Print fill characters
    if (!uFormat.sFormat.bLeftAlign)
    {
        for (size_t i = 0; i < nWidth; i++)
        {
            write(' ');
        }
    }

    print(strText);

    // Print fill characters
    if (uFormat.sFormat.bLeftAlign)
    {
        for (size_t i = 0; i < nWidth; i++)
        {
            write(' ');
        }
    }
}



//-------------------------------------------------------------------
// print
void CEOS_TxStream::print(unsigned int nValue,
                          uint8 nBase,
                          char chLetterBase,
                          size_t nWidth,
                          UFormat uFormat)
{
    size_t nLength = length(nValue,nBase);

    // additional signs?
    if (uFormat.sFormat.bMinusSign)
    {
        nLength++;
    }

    if (nWidth < nLength)
    {
        nWidth = 0;
    }
    else
    {
        nWidth -= nLength;
    }

    // Print fill characters
    if (!uFormat.sFormat.bLeftAlign)
    {
        if (uFormat.sFormat.bZeroFill)
        {
            if (uFormat.sFormat.bMinusSign)
            {
                write('-');
            }
            for (size_t i = 0; i < nWidth; i++)
            {
                write('0');
            }
        }
        else
        {
            for (size_t i = 0; i < nWidth; i++)
            {
                write(' ');
            }
            if (uFormat.sFormat.bMinusSign)
            {
                write('-');
            }
        }
    }
    else if (uFormat.sFormat.bMinusSign)
    {
        write('-');
    }

    print(nValue,nBase,chLetterBase);

    if (uFormat.sFormat.bLeftAlign)
    {
        for (size_t i = 0; i < nWidth; i++)
        {
            write(' ');
        }
    }
}



//-------------------------------------------------------------------
// vprintf
void CEOS_TxStream::vprintf(const char* pFormat,
                            va_list& args)
{
    size_t nWidth;
    // Default precision is 6
    uint8 nPrecision = 6;
    UFormat uFormat;

    // As long the format string does not end, process it
    for (; *pFormat != '\0'; pFormat++)
    {
        // A formation starts with %
        if (*pFormat == '%')
        {
            // Next character
            pFormat++;
            // Assume width 0 and format left
            nWidth = 0;
            uFormat.byte = 0x00;

            if (*pFormat == '\0')
            {
                break;
            }
            // a 2nd % will produce the standard % sign
            if (*pFormat == '%')
            {
                write('%');
                continue;
            }
            // - starts to have left alignement
            if (*pFormat == '-')
            {
                pFormat++;
                uFormat.sFormat.bLeftAlign = true;
            }
            // eat up zeros
            while (*pFormat == '0')
            {
                pFormat++;
                uFormat.sFormat.bZeroFill = true;
            }
            // Generate the width
            for ( ; (*pFormat >= '0') && (*pFormat <= '9'); pFormat++)
            {
                nWidth *= 10;
                nWidth += *pFormat - '0';
            }
            // floats can have precision
            if (*pFormat == '.')
            {
                // We have a precision given, so ignore the old one
                nPrecision = 0;
                pFormat++;
                // here starts precision
                for ( ; (*pFormat >= '0') && (*pFormat <= '9'); pFormat++)
                {
                    nPrecision *= 10;
                    nPrecision += *pFormat - '0';
                }
            }
            // Differnet types of print
            if (*pFormat == 's')
            {
                const char* strString = (char *)va_arg( args, int );
                // Check for not null pointer?
                print(strString ? strString : "(null)",
                      nWidth,
                      uFormat);
                continue;
            }
            else if (*pFormat == 'd')
            {
                int nValue = va_arg(args,int);
                if (nValue < 0)
                {
                    uFormat.sFormat.bMinusSign = true;
                    nValue = -nValue;
                }
                print((unsigned int)nValue,
                      10,
                      'a',
                      nWidth,
                      uFormat);
                continue;
            }
            else if (*pFormat == 'x')
            {
                unsigned int nValue = va_arg(args,unsigned int);
                print(nValue,
                      16,
                      'a',
                      nWidth,
                      uFormat);
                continue;
            }
            else if (*pFormat == 'X')
            {
                unsigned int nValue = va_arg(args,unsigned int);
                print(nValue,
                      16,
                      'A',
                      nWidth,
                      uFormat);
                continue;
            }
            else if (*pFormat == 'u')
            {
                unsigned int nValue = va_arg(args,unsigned int);
                print(nValue,
                      10,
                      'a',
                      nWidth,
                      uFormat);
                continue;
            }
            else if (*pFormat == 'c')
            {
                // chars are converted to int when pushed on the stack
                char strDummy[2];
                strDummy[0] = (char)va_arg(args, int);
                strDummy[1] = '\0';
                print(strDummy,
                      nWidth,
                      uFormat);
                continue;
            }
            else
            {
                // Unknown format
                write('?');
            }
        }
        else
        {
            write(*pFormat);
        }
    }
}



//-------------------------------------------------------------------
// printf
void CEOS_TxStream::printf(const char* strFormat,
                           ...)
{
    va_list args;
    va_start(args,strFormat);
    vprintf(strFormat,
            args);
    va_end(args);
}


//-------------------------------------------------------------------
// operator <<
CEOS_TxStream& CEOS_TxStream::operator << (char ch)
{
    write(ch);
    return *this;
}


//-------------------------------------------------------------------
// operator <<
CEOS_TxStream& CEOS_TxStream::operator << (bool bValue)
{
    if (bValue)
    {
        write('1');
    }
    else
    {
        write('0');
    }
    return *this;
}


//-------------------------------------------------------------------
// operator <<
CEOS_TxStream& CEOS_TxStream::operator << (int nValue)
{
    print(nValue,m_nBase,m_chLetterBase,m_nWidth,m_uFormat);
    return *this;
}


//-------------------------------------------------------------------
// operator <<
CEOS_TxStream& CEOS_TxStream::operator << (double dblValue)
{
    uint8 nWidth = m_nWidth;
    if (nWidth < m_nPrecision+1)
    {
        nWidth = m_nPrecision + 1;
    }
    print((int)dblValue,10,'A',nWidth-m_nPrecision-1,m_uFormat);
    write('.');
    // Cut off the digits > 1
    dblValue -= (int)dblValue;
    for (uint8 i = 0; i < m_nPrecision; i++)
    {
        dblValue *= 10;
    }
    UFormat uFormat = m_uFormat;
    uFormat.sFormat.bZeroFill = true;
    print((int)dblValue,10,'A',m_nPrecision,uFormat);
    return *this;
}


//-------------------------------------------------------------------
// operator <<
CEOS_TxStream& CEOS_TxStream::operator << (const char* strText)
{
    print(strText);
    return *this;
}


//-------------------------------------------------------------------
// manipulator endl
CEOS_TxStream& endl(CEOS_TxStream& EOS_TxStream)
{
    EOS_TxStream.write('\n');
    return EOS_TxStream;
}


//-------------------------------------------------------------------
// manipulator hex
CEOS_TxStream& hex(CEOS_TxStream& EOS_TxStream)
{
    EOS_TxStream.m_nBase = 16;
    EOS_TxStream.m_chLetterBase = 'A';
    return EOS_TxStream;
}


//-------------------------------------------------------------------
// manipulator dec
CEOS_TxStream& dec(CEOS_TxStream& EOS_TxStream)
{
    EOS_TxStream.m_nBase = 10;
    return EOS_TxStream;
}


//-------------------------------------------------------------------
// manipulator oct
CEOS_TxStream& oct(CEOS_TxStream& EOS_TxStream)
{
    EOS_TxStream.m_nBase = 8;
    return EOS_TxStream;
}


//-------------------------------------------------------------------
// manipulator bin
CEOS_TxStream& bin(CEOS_TxStream& EOS_TxStream)
{
    EOS_TxStream.m_nBase = 2;
    return EOS_TxStream;
}


//-------------------------------------------------------------------
// manipulator left
CEOS_TxStream& left(CEOS_TxStream& EOS_TxStream)
{
    EOS_TxStream.m_uFormat.sFormat.bLeftAlign = true;
    return EOS_TxStream;
}


//-------------------------------------------------------------------
// manipulator right
CEOS_TxStream& right(CEOS_TxStream& EOS_TxStream)
{
    EOS_TxStream.m_uFormat.sFormat.bLeftAlign = false;
    return EOS_TxStream;
}


//-------------------------------------------------------------------
// manipulator width
CEOS_TxStream& CEOS_TxStream::width(uint8 nWidth)
{
    m_nWidth = nWidth;
    return *this;
}


//-------------------------------------------------------------------
// manipulator precision
CEOS_TxStream& CEOS_TxStream::precision(uint8 nPrecision)
{
    m_nPrecision = nPrecision;
    return *this;
}


//-------------------------------------------------------------------
// manipulator reset
CEOS_TxStream& CEOS_TxStream::reset()
{
    m_nWidth = 0;
    m_nPrecision = 0;
    m_nBase = 10;
    m_uFormat.byte = 0x00;
    return *this;
}
